'''
use existing python and R code to update the implemneted function to work with pyspark.Pandas dataframes for scalability
all the matrix operations are built into the pyspark pandas class; any additional or more complex operations may need to be written using Spark directly rather than the pandas buffer
Moreover, similar calculations and steps can be followed to use the code without distributive computation using regular pandas library
'''
import numpy as np
from pyspark.sql.functions import expr
from scipy.special._precompute.expn_asy import x
from scipy.stats import norm, chi2
from pyspark.sql import functions as F, SparkSession
from pyspark.sql.types import FloatType, StructType, StructField
from scipy import integrate #can not use scipy :((((((





def getGFisherGM(D, w, M, p_type="two"):
    '''
    Calculate a Fisher information matrix (GM) in the context of network analysis.

    **Inputs:**

    * D: A PySpark Series representing degrees of freedom for each node in the network.
    * w: A PySpark Series of weights associated with the nodes in the network.
    * M: The network adjacency matrix as a PySpark DataFrame.
    * p_type (optional): A string specifying the type of p-values used ("two" for two-sided, "one" for one-sided). Defaults to "two".

    **Output:**

    * The function returns the calculated Fisher information matrix (GM).

    **Example Usages:**

    ```
    D = df['degrees_of_freedom']  # PySpark Series for degrees of freedom
    w = df['weights']  # PySpark Series for weights
    M = spark.createDataFrame(np.random.randn(3, 3).tolist())  # Example matrix M as a PySpark DataFrame
    p_type = "two"  # Two-sided input p-values
    result = getGFisherGM(D, w, M, p_type)
    print(result)

    '''

    n = M.count()

    def integrand(x, y, m, p_type):
        if p_type == "two":
            return qchisq(pchisq(x ** 2, df=1), df=y) * dnorm(x) * (m ** 2 / 2) * (x ** 2 - 1)
        else:
            pnormX = F.when(F.isnan(F.pnorm(x)), F.lit(0)).otherwise(F.pnorm(x))
            return qchisq(pnormX, df=y) * dnorm(x) * m * x

    def qchisq(p, df):
        return chi2.ppf(p, df)

    def pchisq(q, df):
        return chi2.cdf(q, df)

    def dnorm(x):
        return norm.pdf(x)

    if p_type == "two":
        coeff2_expr = F.udf(lambda x, y, m: integrand(x, y, m, p_type), returnType=FloatType())
        coeff2 = np.array([coeff2_expr(F.lit(x), F.lit(y), M[i][j]).cast(FloatType()) for i, y in enumerate(D) for j in range(n)])
        coeff2 = coeff2.reshape((n, n))

        coeff4_expr = F.udf(lambda x, y, m: integrand(x, y, m, p_type), returnType=FloatType())
        coeff4 = np.array([coeff4_expr(F.lit(x), F.lit(y), M[i][j]).cast(FloatType()) for i, y in enumerate(D) for j in range(n)])
        coeff4 = coeff4.reshape((n, n))

        coeff6_expr = F.udf(lambda x, y, m: integrand(x, y, m, p_type), returnType=FloatType())
        coeff6 = np.array([coeff6_expr(F.lit(x), F.lit(y), M[i][j]).cast(FloatType()) for i, y in enumerate(D) for j in range(n)])
        coeff6 = coeff6.reshape((n, n))

        coeff8_expr = F.udf(lambda x, y, m: integrand(x, y, m, p_type), returnType=FloatType())
        coeff8 = np.array([coeff8_expr(F.lit(x), F.lit(y), M[i][j]).cast(FloatType()) for i, y in enumerate(D) for j in range(n)])
        coeff8 = coeff8.reshape((n, n))

        GM = M ** 2 / 2 * coeff2 @ coeff2.T + M ** 4 / 24 * coeff4 @ coeff4.T + M ** 6 / 720 * coeff6 @ coeff6.T + M ** 8 / 40320 * coeff8 @ coeff8.T
        GM = np.diag(np.sqrt(np.multiply(D, 2)) * w) @ np.corrcoef(GM) @ np.diag(np.sqrt(np.multiply(D, 2)) * w)
    else:
        coeff3_expr = F.udf(lambda x, y, m: integrand(x, y, m, p_type), returnType=FloatType())
        coeff3 = np.array([coeff3_expr(F.lit(x), F.lit(y), M[i][j]).cast(FloatType()) for i, y in enumerate(D) for j in range(n)])
        coeff3 = coeff3.reshape((n, n))

        coeff4_expr = F.udf(lambda x, y, m: integrand(x, y, m, p_type), returnType=FloatType())
        coeff4 = np.array([coeff4_expr(F.lit(x), F.lit(y), M[i][j]).cast(FloatType()) for i, y in enumerate(D) for j in range(n)])
        coeff4 = coeff4.reshape((n, n))

        coeff2_expr = F.udf(lambda x, y, m: integrand(x, y, m, p_type), returnType=FloatType())
        coeff2 = np.array([coeff2_expr(F.lit(x), F.lit(y), M[i][j]).cast(FloatType()) for i, y in enumerate(D) for j in range(n)])
        coeff2 = coeff2.reshape((n, n))

        coeff1_expr = F.udf(lambda x, y, m: integrand(x, y, m, p_type), returnType=FloatType())
        coeff1 = np.array([coeff1_expr(F.lit(x), F.lit(y), M[i][j]).cast(FloatType()) for i, y in enumerate(D) for j in range(n)])
        coeff1 = coeff1.reshape((n, n))

        GM = M * coeff1 @ coeff1.T + M ** 2 / 2 * coeff2 @ coeff2.T + M ** 3 / 6 * coeff3 @ coeff3.T + M ** 4 / 24 * coeff4 @ coeff4.T
        GM = np.diag(np.sqrt(2 * D) * w) @ np.corrcoef(GM) @ np.diag(np.sqrt(2 * D) * w)

    return GM

# Initialize SparkSession
spark = SparkSession.builder \
    .appName("Fisher Information Matrix Calculation") \
    .getOrCreate()

# Example data
D_values = [1, 2, 3]  # Values for degrees of freedom
w_values = [0.5, 0.3, 0.2]  # Weights
M_data = [
    [1.0, 0.5, 0.2],
    [0.5, 1.0, 0.3],
    [0.2, 0.3, 1.0]
]

# Convert M_data to a PySpark DataFrame
schema = StructType([StructField(str(i), FloatType(), True) for i in range(len(M_data[0]))])
M_df = spark.createDataFrame(M_data, schema)

# Create PySpark Series for D and w
D_series = spark.createDataFrame([(float(x),) for x in D_values], ['degrees_of_freedom'])
w_series = spark.createDataFrame([(float(x),) for x in w_values], ['weights'])

# Call the getGFisherGM function
result = getGFisherGM(D_series, w_series, M_df)

# Show the result
result.show()

# Stop SparkSession
spark.stop()