'''
this file contains code to generate all the necessary figures for the report based on the results obtains in the csv files
'''
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
def createScatterPlot(data, x_label, y_label, imgName):
    '''
    create a scatter plot of the given data and save it into a png
    :param data: input data to be scattered
    :param x_label: name of the x axis
    :param y_label: name of the y axis
    :param imgName: name of the file in which the image will be saved
    :return: void

    Example:
    #assuming that the data is being read from a csv using a pandas dataframe
    df = pd.read_csv("pythonTimes.csv").dropna()
    first_column = df.iloc[:, 0].values
    createScatterPlot(first_column, "Number of the Run", "Processing Time", "pythonPandas1000Scatter.png")
    '''
    # Plot all points except the last one in blue
    plt.scatter(range(len(data) - 1), data[:-1], color='blue')

    # Plot the last point in red
    plt.scatter(len(data) - 1, data[-1], color='red')
    plt.xlabel(x_label)
    plt.ylabel(y_label)
    plt.title("Scatter Plot")
    plt.show()

    # Save the plot as a PNG file
    plt.savefig(imgName)

def createBarPlot (data, x_labels, y_label, imgName):
   '''
   create a bar plot given some data
   :param data: data to plot on the x axis
   :param x_labels: labels for the x axis
   :param y_label: labels for the y-axis
   :param imgName: name of the image into which the plot will be saved
   :return: void

   Example:
   Assume that the data is stored in a csv file and accessed through a pandas dataframe   '''
   # Plot the bar plot
   plt.bar(range(len(data)), data)

   # Set custom x-axis labels
   plt.xticks(range(len(x_labels)), x_labels)

   plt.xlabel(x_labels)
   plt.ylabel(y_label)
   plt.show()
   plt.savefig(imgName)

def multiBarPlots(y1, y2, y3, y_label, imgName):
    # Create subplots
    x = range(len(y1))
    fig, axs = plt.subplots(1, 3, figsize=(15, 5))  # 1 row, 3 columns

    # Bar plots with labels
    axs[0].bar(x, y1, color='blue')
    axs[0].set_title('10x10 Matrix')
    #axs[0].set_xlabel([10, 100, 1000])
    axs[0].set_xticks([0, 1, 2, 3])
    axs[0].set_xticklabels(['R', 'Python Numpy', 'Python Pandas', 'Spark'])
    axs[0].set_ylabel(y_label)

    axs[1].bar(x, y2, color='red')
    axs[1].set_title('100x100 Matrix')
    axs[1].set_xticks([0, 1, 2, 3])
    axs[1].set_xticklabels(['R', 'Python Numpy', 'Python Pandas', 'Spark'])
    axs[1].set_ylabel(y_label)

    axs[2].bar(x, y3, color='green')
    axs[2].set_title('1000x1000 Matrix')
    axs[2].set_xticks([0, 1, 2, 3])
    axs[2].set_xticklabels(['R', 'Python Numpy', 'Python Pandas', 'Spark'])
    axs[2].set_ylabel(y_label)


    # Adjust layout
    plt.tight_layout()

    plt.show()

    # Save plot
    plt.savefig(imgName)


'''
    axs[3].bar(x, y4, color='orange')
    axs[3].set_title('PySpark SQL')
    axs[3].set_xticks([0, 1, 2])
    axs[3].set_xticklabels(['10', '100', '1000'])
    axs[3].set_ylabel(y_label)

'''

def multiScatterPlots(y1, y2, y3, imgName):
    x = np.arange(0, 100, 1)
    # Create subplots
    fig, axs = plt.subplots(1, 3, figsize=(15, 5))  # 1 row, 3 columns

    # Scatter plots
    axs[0].scatter(x, y1, color='blue')
    axs[0].set_title('Computation Times for 10x10 Matrix')

    axs[1].scatter(x, y2, color='red')
    axs[1].set_title('Computation Times for 100x100 Matrix')

    axs[2].scatter(x, y3, color='green')
    axs[2].set_title('Computation Times for 1000x1000 Matrix')

    # Adjust layout
    plt.tight_layout()
    plt.savefig(imgName)
    # Show plots
    plt.show()


'''
df = pd.read_csv("/home/taya/distributedProcessing/results/overallAvgs.csv").dropna()
y1 = df.iloc[0].values
y2 = df.iloc[1].values
y3 = df.iloc[2].values
y4 = df.iloc[2].values
y11 = y1[1:4]
y22 = y2[1:4]
y33 = y3[1:4]
y44 = y4[1:4]

multiBarPlots(y11, y22, y33, y44, "Processing Time", "avgBars.png")

#scatter plots for spark data
sparkDf = pd.read_csv("/home/taya/distributedProcessing/results/sparkTimes.csv")
multiScatterPlots(sparkDf.iloc[:, 1],
                  sparkDf.iloc[:, 2],
                  sparkDf.iloc[:, 3],
                  "spark5.png")
'''

df = pd.read_csv("/home/taya/distributedProcessing/results/overallAvgs.csv")

y1 = df.iloc[0]
y11 = np.array(y1[1:])
y2 = df.iloc[1]
y22 = np.array(y2[1:])
y3 = np.array(df.iloc[2])
y33 = np.array(y3[1:])
multiBarPlots(y11, y22, y33, ["Processing Times"], "compPlot.png")