source("/home/taya/ALS_Analysis/Code/GFisher_Apply_Integrative_Test_code/getwts_WIP.R")
source("/home/taya/ALS_Analysis/Code/GFisher_Apply_Integrative_Test_code/geno_data_processing.R")
source("/home/taya/ALS_Analysis/Code/GFisher_Apply_Integrative_Test_code/GFisher_v1.R")

start_time <- Sys.time()
library(vcfR)
vcfFile = "/home/taya/Downloads/condensed_vcf_files_chr21_filtered_50samples_sn.vcf.gz"
phenoFile = "/home/taya/ALS_Analysis/Code/getwts_real_example/Samples_50_phenotypes.csv"
vcf <- read.vcfR(vcfFile)

# Use CXADR as an example gene
gene_start = 17513043
gene_end = 17636262

geneVCF = vcf[getPOS(vcf)>=gene_start & getPOS(vcf)<=gene_end,]
# default genotype matrix from vcf file is SNP x Samples
G = extract.gt(geneVCF, element = 'GT', as.numeric = TRUE)
# need to convert to Samples x SNP format
G = t(G)
# missing imputed as 0
G[is.na(G)] = 0

# calculate MAF
maf = colMeans(G)/2
# remove MAF=0 SNPs
G = as.matrix(G[,maf>0,drop=F])
# remove SNP with sd=0
G = G[,which(apply(G,2,sd)>0),drop=F]

# filter genotype based on ld and MAC
Gfilter = filter_genotype_mac_ld(G,mac.thresh=5,ld.thresh=0.99)
maf = Gfilter$MAF
G = Gfilter$G
write.table(G, file = "matrix.txt", sep = "\t", row.names = FALSE, col.names = FALSE)


snpN = dim(G)[2]
sampleN = dim(G)[1]

# Assume there is an annotation ~ N(0,1) and it determines causality
anno = rnorm(snpN)
causalprob = rje::expit(rje::logit(0.0275) + log(20) * anno)
isCausal = rbinom(snpN,1,causalprob)

# Assume the coefficient beta is related to log10(MAF)
b = -0.4 * log10(maf)

pheno = read.csv(phenoFile)

# Assume we include gender and age of onset as covariates
X = as.matrix(pheno[,c("gender_numeric","age_of_onset")])
Y = pheno$Diagnosis_numeric
# Get null model
nullmod = get_H(X=X,Y=Y,trait="binary")
# Get optimal weights generated by beta and PI (causal probability)
mywts = Optimal_Weights_WIP(function(x)x^2, G, B=b, PI=causalprob, trait="binary", nullmod=nullmod)
# 4 optimal weights
BE_wts_N = mywts$wts_BE_N
print(BE_wts_N)
APE_wts_N = mywts$wts_APE_N
BE_wts_sparse = mywts$wts_BE_sparse
APE_wts_sparse = mywts$wts_APE_sparse

# Calculate statistics using different weights
Y0 = nullmod$Y0
Gtilde = t(t(sqrt(Y0*(1-Y0))*G))
GHhalf = t(Gtilde)%*%nullmod$Hhalf
GHG = t(Gtilde)%*%Gtilde - GHhalf%*%t(GHhalf) #omega = diag(GHG)

# Z score (T_M)
score = t(G)%*%(Y-Y0)
Zscore = score/(sqrt(diag(GHG))) # score/omega-1/2
Tstat = sum(Zscore^2)
Tstat_causalOnly = sum(Zscore[which(isCausal==1),]^2)
Tstat_wts_BE_N = sum(BE_wts_N*Zscore^2)
Tstat_wts_APE_N = sum(APE_wts_N*Zscore^2)
Tstat_wts_BE_sparse = sum(BE_wts_sparse*Zscore^2)
Tstat_wts_APE_sparse = sum(APE_wts_sparse*Zscore^2)

c(Tstat,Tstat_causalOnly,Tstat_wts_BE_N,Tstat_wts_APE_N,Tstat_wts_BE_sparse,Tstat_wts_APE_sparse)

# Calculate p-value of the gene
corM = cor(G)
d = rep(1,snpN)
p_noWeight = p.GFisher(q=Tstat,df=d,w=1,M=corM)
p_causalOnly = p.GFisher(q=Tstat_causalOnly,df=d,w=isCausal,M=corM)
p_BE_N = p.GFisher(q=Tstat_wts_BE_N,df=d,w=as.vector(BE_wts_N),M=corM)
p_APE_N = p.GFisher(q=Tstat_wts_APE_N,df=d,w=as.vector(APE_wts_N),M=corM)
p_BE_sparse = p.GFisher(q=Tstat_wts_BE_sparse,df=d,w=as.vector(BE_wts_sparse),M=corM)
p_APE_sparse = p.GFisher(q=Tstat_wts_APE_sparse,df=d,w=as.vector(APE_wts_sparse),M=corM)

c(p_noWeight,p_causalOnly,p_BE_N,p_APE_N,p_BE_sparse,p_APE_sparse)
end_time <- Sys.time()
print(start_time-end_time)


