#basic matrix operations on py.spark
#such implementations will be needed for refining our algorithms to work with clusters
from pyspark.sql import SparkSession
from pyspark.sql import Row
from pyspark.sql.functions import col, expr

spark = SparkSession.builder \
    .appName("MatrixOperations") \
    .getOrCreate()

# Example matrices
matrix1 = [(1, 2, 3),
           (4, 5, 6),
           (7, 8, 9)]

matrix2 = [(9, 8, 7),
           (6, 5, 4),
           (3, 2, 1)]

# Create DataFrame for each matrix
df1 = spark.createDataFrame([Row(*row) for row in matrix1])
df2 = spark.createDataFrame([Row(*row) for row in matrix2])

# Addition
df_addition = df1.rdd.zip(df2.rdd).map(lambda x: [sum(pair) for pair in zip(x[0], x[1])]).toDF()
# Print addition result
print("Matrix Addition:")
df_addition.show()

# Subtraction
df_subtraction = df1.rdd.zip(df2.rdd).map(lambda x: [pair[0] - pair[1] for pair in zip(x[0], x[1])]).toDF()
# Print subtraction result
print("Matrix Subtraction:")
df_subtraction.show()

# Transpose
df_transpose = df1.select(*[col(c).alias('row_' + str(i)) for i, c in enumerate(df1.columns)])
# Print transpose result
print("Matrix Transpose:")
df_transpose.show()


#stop the clusters
spark.stop()