'''
this file showcases the connection between Hail and Spark by performing a simple standard deviation calculation
'''
import hail as hl
import pyspark.pandas as ps


hl.init(log="hailSparkCalculation.log")
#create a hail matrix

t = hl.utils.range_table(10)
t = t.annotate(a = t.idx, b = t.idx * t.idx, c = hl.str(t.idx))
t.describe()

#perform the stats calculation
print("Row a values are as follows")
t.a.show()
stats = t.aggregate(hl.agg.stats(t.a))
print("The Hail statistics for row  a are as follows: ")
print(stats)

#convert a Hail matrix into pySpark
print("The PySpark statistics for row a are as follows: ")
psA = ps.DataFrame(t.a.collect())
mean = psA[0].mean()
std = psA[0].std(ddof=0)
min = psA[0].min()
max = psA[0].max()
sum = psA[0].sum()
psStats = { "mean": mean,
            "stdev":std,
            "min":min,
            "max":max,
            "sum": sum
}
print(psStats)