'''
this file contains information to calculate the average performance time of calculating covariance between two matrices
all the procedures are repeated 100 times and each of them is timed; the times are recorded into a padnas dataframe which is the used to find the averages and plot all the data
the computations were performed using 10x10, 100x100, and 1000x1000 matrices
'''
import time

import pandas as pd
import pyspark.pandas as ps
import numpy as np
import hail as hl

#create the file for results

#record results into a dataframe
#TODO: add a try catch and create a file if it does not already exist
#TODO: clean up the examples
def createCsv (fileName):
    '''
    create and populate a csv file for future use when writing and reading it
    :param fileName: name of the file to be created
    :return: void; creates the file with a given name with a column names is with values from 1 to 100
    '''
def recordResults(data, fileName, columnName):
    '''
    :param data: input data to be recorded into a csv file
    :param fileName: name of the file containing all the results; make sure that the file already exists even if it is empty
    :param columnName: name of the new column to record the results into
    :return: void; modifies the file

    Example:
    m1a = np.random.randint(1, 101, size=(10, 10)) #10 to the first power size m1
    m1b = np.random.randint(1, 101, size=(10, 10)) #10 to hthe first power size m1

    m2a = np.random.randint(1, 101, size=(100, 100))
    m2b = np.random.randint(1, 101, size=(100, 100))

    m3a = np.random.randint(1, 101, size=(1000, 1000))
    m3b = np.random.randint(1, 101, size=(1000, 1000))

    npTimes1 = numpyCov(m1a, m1b)
    npTimes2 = numpyCov(m2a, m2b)
    npTimes3 = numpyCov(m3a, m3b)
    '''

    #read df from a csv file
    df = pd.read_csv(fileName)
    # Add a new column
    new_column = data
    df[columnName] = new_column
    # Write modified data back to CSV file
    df.to_csv(fileName, index=False)


#calculate the covariance of 2 matrices using numpy
# Sample data as NumPy arrays
def numpyCov(arr1):
    '''
    calculate the times it takes to calculate covariance of two input numpy matrices
    :param arr1: numpy array of any dimension
    :param arr2: numpy array of any dimension
    :return: an array of resulting times from each calculation
    '''
    allTimes = np.array([])
    for i in range(0,100):
        start = time.time()
        cov = np.cov(arr1)
        end = time.time()
        totalProcessing = end - start
        allTimes = np.append(allTimes, totalProcessing)
    return allTimes

#TODO: make sure that these results are comparable?????? with numpy; I think they are now but still need to make sure -> need to see if I need to recalculate those times but it still seems fairly reasonable
def pdCov (df1):
    '''
    calculate the time the covariance of two given matrices; run the operation 100 times
    :param df1: pandas dataframe nxn
    :param df2: pandas dataframe nxn
    :return: a list of all processing times

    Example:
    #generate random numpy arrays of various length to record the times
    m1a = np.random.randint(1, 101, size=(10, 10)) #10 to the first power size m1
    m1b = np.random.randint(1, 101, size=(10, 10)) #10 to hthe first power size m1
    pd1a = pd.DataFrame(m1a)
    pd1b = pd.DataFrame(m1b)

    m2a = np.random.randint(1, 101, size=(100, 100))
    m2b = (np.random.randint(1, 101, size=(100, 100)))
    pd2a = pd.DataFrame(m2a)
    pd2b = pd.DataFrame(m2b)

    m3a = np.random.randint(1, 101, size=(1000, 1000))
    m3b = np.random.randint(1, 101, size=(1000, 1000))
    pd3a = pd.DataFrame(m3a)
    pd3b = pd.DataFrame(m3b)


    pdTimes1 = pdCov(pd1a)
    pdTimes2 = pdCov(pd2a)
    pdTimes3 = pdCov(pd3a)
    recordResults(pdTimes1, "results/pythonTimes.csv", "pdTimes1")
    recordResults(pdTimes2, "results/pythonTimes.csv", "pdTimes2")
    recordResults(pdTimes3, "results/pythonTimes.csv", "pdTimes3")
    '''
    allTimes = np.array([])
    for i in range(0,100):
        start = time.time()
        covariance_matrix = df1.cov()
        end = time.time()
        total = end - start
        allTimes = np.append(allTimes, total)
    return allTimes

#TODO: add calculations with multiple clusters and include the versions used
def psCov (df1):
    '''
    calculate the time it takes to process a pyspark pandas dataframe covariance, repeat the process 100 times and record the values
    :param df1: pyspark dataframe
    :return: list of all time calculations
    '''
    allTimes = np.array([])
    for i in range(0, 100):
        start = time.time()
        covariance_matrix = df1.cov()
        end = time.time()
        total = end - start
        allTimes = np.append(allTimes, total)
    return allTimes

#TODO: i do not think this actually works, the only thing we can do here is get the cov matrix fror simulating the
def hailCov (mt):
    '''

    :param data:
    :return:
    '''

    allTimes = np.array([])
    for i in range(0, 100):
        start = time.time()
        covariance_matrix = mt.aggregate_cols(hl.agg.cov(mt.GT.n_alt_alleles()))
        end = time.time()
        total = end - start
        allTimes = np.append(allTimes, total)
    return allTimes


#generate random numpy arrays of various length to record the times
m1a = np.random.randint(1, 101, size=(10, 10)) #10 to the first power size m1
pd1a = ps.DataFrame(m1a)

m2a = np.random.randint(1, 101, size=(100, 100))
pd2a = ps.DataFrame(m2a)

m3a = np.random.randint(1, 101, size=(1000, 1000))
pd3a = ps.DataFrame(m3a)

#TODO: figure out how many clusters spark is using, what is the CPU, etc -> need to process spark on not one computer bc thing just failed lol

psTimes1 = psCov(pd1a)
recordResults(psTimes1, "../pythonTimes.csv", "psTimes1")
#psTimes2 = psCov(pd2a) #runs out of memory on a single machine :(
#psTimes3 = psCov(pd3a)
#print(psTimes1)
#recordResults(psTimes2, "results/pythonTimes.csv", "psTimes2")
#recordResults(psTimes3, "results/pythonTimes.csv", "psTimes3")

