#https://hail.is/docs/0.2/hail.Table.html#hail.Table.to_spark
#THIS IS THE FILE THAT DOES ALL THE CONVERSION

'''
file to rewrite and run the pipeline using Spark and Hail combination
spark pandas dataframe reference
https://spark.apache.org/docs/latest/api/python/reference/pyspark.pandas/frame.html?highlight=stat#computations-descriptive-stats
'''
import hail as hl
import pyspark.pandas as ps
import exampleHt as eht

hl.init(log="sparkPipeline.log")
ht = hl.read_table("hailMatrices/ht.mt")
ht.show()
ht.describe()
sp = ht.to_spark(flatten=True) #returns a pyspark.sql.dataframe.DataFrame which can be converted into pyspark.pandas if necessary
#However, all the operations can be performed on the sql basis here
#the sql dataframe operations are more similar to those of Hail operations with aggregate, map, collect etc
#pandas, on the other hand, has functionality similar to that of traditional pandas library and offers more way to compute various stat/math values
spPandas = ps.DataFrame(sp)

#now that the dataframes have been created we can utilize Hail or direct ps calculations
#to find the necessary values for p_GFisher calculations
'''
in order to not use numpy, we need to rewrite our implementation of p_GFisher such that it takes in ps.DataFrame
which will require for all the calculations to be rewritten
below I suggest an oversimplified algorithm to calculate the necessary 
general p values and then calculate the individual snp values given the examples provided in hail conversions
'''

pvals, gf, M = eht.findGFM(ht) #may need to rewrite this just because of conversion issues(?)
'''
we are using existing calculations which were perfomed with Hail and converting them into
a datatrype compatible with dist computing
in the future, it would make sense to immidiately transition to spark but combination with Hail for initial processing
should also work well in terms of computational power needed
'''

pvalsPandas = ps.DataFrame(pvals.collect())
gfPandas = ps.DataFrame(gf.collect())#check if this conversion works correctly
print(pvals.collect())
print(pvalsPandas)
print(gfPandas)


'''
Notice that after the pySpark dataframes have been created in lines 43-44 above, they can be used in the new implementation of p_GFisher calculations
Moreover, if it is more convenient, pySpark sql can be used directly after Hail's converstion with the function to_spark()
Below is an example of a simple standard deviation calculation on a given pandas dataframe and its Hail counter part
'''







