'''
this file provide computations analogous to those performed to calculate the covariance of a pyspark sql matrix
'''

from pyspark.sql import SparkSession
from pyspark.sql.functions import rand
import time

# Create a SparkSession
spark = SparkSession.builder \
    .appName("RandomDataFrame") \
    .getOrCreate()

# Define the number of rows and columns
num_rows = 1000
num_cols = num_rows

# Create a list of column names
columns = [f"col{i}" for i in range(1, num_cols+1)]

# Create a DataFrame with random numbers in range 1 to 100
df = spark.range(num_rows).select(*[(rand() * 100).cast("int").alias(col) for col in columns])

# Record processing times
processing_times = []

# Run the process 100 times
for _ in range(100):
    start_time = time.time()

    # Calculate the covariance matrix
    cov = df.cov("col1", "col2")
    end_time = time.time()
    elapsed_time = end_time - start_time
    processing_times.append(elapsed_time)

# Show the processing times
print("Processing times (seconds):", processing_times)
print("The average processing time is", sum(processing_times)/len(processing_times))

# Stop the SparkSession
spark.stop()